import {
	btn,
	checkbox, CheckboxField,
	column,
	comp,
	EntityID, fieldset,
	Filter,
	h3,
	hr,
	menu,
	mstbar,
	router,
	searchbtn,
	t,
	tbar, Window
} from "@intermesh/goui";
import {projectbooktable} from "./ProjectBookTable.js";
import {ProjectBookWindow} from "./ProjectBookWindow.js";
import {ProjectTable, projecttable} from "./ProjectTable.js";
import {ProjectWindow} from "./ProjectWindow.js";
import {ProjectDetail} from "./ProjectDetail.js";
import {statusfilter, StatusFilter} from "./StatusFilter.js";
import {client, filterpanel, jmapds, MainThreeColumnPanel} from "@intermesh/groupoffice-core";


export class MainPanel extends MainThreeColumnPanel {
	private projectTable?: ProjectTable;
	private statusFilter?: StatusFilter;

	/**
	 * Used to track if module has been initialized. Then filter changes will be updating the data.
	 * @private
	 */
	private loaded: boolean = false;

	protected east!: ProjectDetail;
	private showMineOnlyCB!: CheckboxField;

	constructor() {
		super("projects3");
	}

	createWest() {

		return comp({
			width: 260,
			itemId: "west",
			stateId: "projects3-west",
			cls: "scroll"
		},
			tbar({
				cls: "border-bottom"
				},
				h3(t("Books")),
				'->',
				btn({
					title: "Add",
					icon: "add",
					handler: () => {
						const d = new ProjectBookWindow();
						d.show();
					}
				}),
				this.showCenterButton()
			),

			projectbooktable({
				cls: "no-row-lines",
				headers: false,
				fitParent: true,
				columns: [
					// checkboxselectcolumn(),
					column({
						id: "name"
					}),
					column({
						sticky: true,
						width: 32,
						id: "btn",
						renderer: (columnValue: any, record, td, table, rowIndex) => {

							return btn({
								icon: "more_vert",
								menu: menu({},
									btn({
										icon: "edit",
										text: t("Edit"),
										handler: async (btn) => {
											const book = table.store.get(rowIndex)!;
											const d = new ProjectBookWindow();
											await d.load(book.id);
											d.show();
										}
									}),
									hr(),
									btn({
										icon: "delete",
										text: t("Delete"),
										handler: async (btn) => {
											const book = table.store.get(rowIndex)!;
											jmapds("ProjectBook").confirmDestroy([book.id]);
										}
									})

									)
							})
						}
					}),

				],
				listeners: {
					render: async (table) => {
						await table.store.load();

						let record = table.store.find((value, index, obj) => {
							return value.isVisible;
						});

						if(record) {
							table.rowSelection!.add(record);

							this.showMineOnlyCB.value = record.filter?.showMineOnly;
						}

					}
				},
				rowSelectionConfig: {
					multiSelect: false,
					listeners: {

						selectionchange: (tableRowSelect) => {

							const bookIds = tableRowSelect.getSelected().map((row) => row.id);

							this.projectTable!.store.setFilter("book", {bookId: bookIds[0]})

							jmapds("ProjectBook").single(bookIds[0]).then(async(book) => {
								// setting the book will apply selected statuses causing as "selectionchange" event.
								// this event is buffered so it will be delayed.
								this.statusFilter!.setBook(book!);
								this.statusFilter?.applyToProjectStore(this.projectTable!.store);
								await this.projectTable!.store.load();
								this.loaded = true;

							})

							// save which book is selected
							const bookStore = tableRowSelect.list.store;

							bookStore.data.forEach((b) => {
								const isVisible = b.id == bookIds[0];

								if(b.isVisible != isVisible) {
									jmapds("ProjectBook").update(b.id, {isVisible: isVisible});
								}
							});
						}
					}
				}
			}),

			fieldset({},
				this.showMineOnlyCB = checkbox({
					type: "switch",
					label: t("Show mine only"),
					name: "showMineOnly",
					listeners: {
						setvalue: (field, newValue, oldValue) => {

							this.projectTable!.store.setFilter("showMineOnly", newValue ? {isResource: client.user.id} : undefined);

							const book = this.statusFilter!.getBook();
							if(!book) {
								return;
							}

							const filter = book.filter ?? {};
							filter.showMineOnly = newValue;


							if(this.loaded) {
								void this.projectTable!.store.load();
							}

							jmapds("ProjectBook").update(book.id, {filter}).catch(e => void Window.error(e));

						}
					}
				})
			),

			tbar({
					cls: "border-bottom"
				},

				checkbox({
					listeners: {
						change: (field, newValue, oldValue) => {

							if (newValue) {
								this.statusFilter!.rowSelection!.selectAll();
							} else {
								this.statusFilter!.rowSelection!.clear();
							}

						}
					}
				}),

				h3(t("Statuses")),

			),
			this.statusFilter = statusfilter({
				rowSelectionConfig: {
					multiSelect: false,
					listeners: {

						selectionchange: (tableRowSelect) => {

							this.statusFilter?.applyToProjectStore(this.projectTable!.store);

							if(this.loaded) {
								void this.projectTable!.store.load();
							}

						}
					}
				}
			}),

			filterpanel({
				entityName: "Project3",
				store: this.projectTable!.store
			})
		)
	}


	createCenter() {

		this.projectTable = projecttable({
			stateId: "projects3-table-main",
			scrollLoad: true,
			fitParent: true,

			listeners: {
				rowdblclick:(list, storeIndex, row, ev) => {
					const d = new ProjectWindow();
					d.show();
					void d.load(list.store.get(storeIndex)!.id);
				},

				delete: async (tbl) => {
					const ids = this.projectTable!.rowSelection!.getSelected().map(row => row.record.id);
					await jmapds("Project3")
						.confirmDestroy(ids);
				}
			},

			rowSelectionConfig: {
				listeners: {
					selectionchange:rowSelect => {
						if(rowSelect.getSelected().length) {
							router.goto("projects3/" + rowSelect.getSelected()[0].record.id);
						}
					}
				}
			}
		});



		return comp({
			flex: 1,
			cls: 'vbox active', //for mobile view this is active
		},
			tbar({
					cls: "border-bottom"
				},

				this.showWestButton(),

				'->',

				searchbtn({
					listeners: {
						input: (searchBtn, text) => {
							this.projectTable!.store.setFilter("search", {text: text});
							void this.projectTable!.store.load();
						}
					}
				}),

				btn({
					title: "Add",
					text: t("Add"),
					cls: "filled primary",
					icon: "add",
					handler: () => {
						const d = new ProjectWindow();
						d.form.value = {
							bookId: this.projectTable!.store.getFilter("book").bookId
						};
						d.show();
					}
				}),

				mstbar({table: this.projectTable!},
					"->",
					btn({
						icon: "delete",
						handler: async (btn) => {

							const ids = this.projectTable!.rowSelection!.getSelected().map(row => row.record.id);

							const result = await jmapds("Project3")
								.confirmDestroy(ids);

							if(result != false) {
								btn.parent!.hide();
							}

						}
					})
				),
			),

			comp({flex: 1, cls: "scroll bg-lowest"},
				this.projectTable
			)
		)
	}

	protected createEast() {
		const detail = new ProjectDetail();
		detail.width = 500;
		detail.itemId = "detail";
		detail.stateId = "projects3-detail";

		detail.toolbar.items.insert(0,this.showCenterButton());
		return detail;
	}

	setProjectId(projectId:EntityID) {
		void this.east.load(projectId);
		this.activatePanel(this.east);
	}
}

