import {
	btn,
	comp,
	Component,
	ContainerFieldValue,
	DateInterval,
	DisplayField,
	displayfield,
	fieldset,
	Form,
	form,
	Format, h3,
	t, tbar
} from "@intermesh/goui";
import {Chart, client, jmapds, router} from "@intermesh/groupoffice-core";
import {InvoiceWindow} from "./InvoiceWindow.js";

export class ProjectFinance extends Component {
	private form: Form<ContainerFieldValue>;
	private projectId?: number;
	private documentId?: number;
	constructor() {
		super();

		this.cls = "project-finance";

		this.items.add(

			this.form = form({
					cls: "label-align-left",
					listeners:{
						beforesetvalue:(field, e) => {

							// calculate some extra form values before setting the form
							if(!e.value.quotedMinutes) {
								e.value.quotedMinutes = e.value.budgetMinutes;
							}
							e.value.tasksTotal = e.value.tasksTodo + e.value.tasksCompleted;
							e.value.billableTotal = e.value.quoteAmount + e.value.billableFee;
							e.value.outstanding = e.value.quoteAmount + e.value.billableFee - e.value.invoicedAmount;
							e.value.profit = e.value.quoteAmount + e.value.billableFee - e.value.quoteCosts - e.value.feeCosts;
							e.value.totalHours = e.value.bookedMinutes + e.value.billableMinutes;

							this.drawHoursChart(e.value);
							this.drawBillableChart(e.value);
							this.drawProfitChart(e.value);
							this.drawTaskChart(e.value);
						}
					}
				},
				tbar({}, displayfield({escapeValue: false, tagName: "a", name: "projectId", renderer: async (v, field) => {
					const project = (await jmapds("Project3").single(v))!;

					field.el.attr("href", "#projects3/" + v)

						return '<h3>' + t("Project", "business", "projects3") + ": " + project.number + " " + project.name + '</h3>';
					}})),

				comp({cls: "project-finance-graphs"},


				fieldset({legend: "Hours", cls: "vbox gap"},

					comp({tagName:"canvas" , id: "project-finance-hour-chart"}),

					displayfield({
						cls: "right label-left",
						name: "quotedMinutes",
						label: t("Estimated hours"),
						renderer:(v) => {
							if(!v) {
								return "";
							}
							const d = DateInterval.createFromFormat(v, "j")!;
							return d.format("h:I");
						}
					}),

					displayfield({
						cls: "right border-bottom",
						name: "bookedMinutes",
						label: t("Actual hours"),
						renderer:(v, field) => {

							if(!v) {
								return "";
							}

							const quotedMinutesField = field.previousSibling() as DisplayField;

							field.el.classList.toggle("warning", v > quotedMinutesField.value!);

							const d = DateInterval.createFromFormat(v, "j")!;
							return d.format("h:I");
						}
					}),

					displayfield({
						cls: "right border-bottom",
						name: "billableMinutes",
						label: t("Billable hours"),
						value: 0,
						renderer:(v) => {
							if(!v) {
								return "";
							}
							const d = DateInterval.createFromFormat(v, "j")!;
							return d.format("h:I");
						}
					}),

					displayfield({
						cls: "right",
						name: "totalHours",
						label: t("Total"),
						renderer:(v) => {
							if(!v) {
								return "";
							}
							const d = DateInterval.createFromFormat(v, "j")!;
							return d.format("h:I");
						}
					}),

					comp({flex: 1}), // fill space till bottom in hbox layout

					btn({
						cls: "outlined",
						text: t("View time entries"),
						handler: () => {
							let url = "registration/list/" + this.projectId + "/";

							router.goto(url);
						}

					})
				),


				fieldset({legend: "Tasks", cls: "vbox gap"},

					comp({tagName:"canvas" , id: "project-finance-task-chart"}),

					displayfield({
						cls: "right label-left",
						name: "tasksCompleted",
						label: t("Completed"),
					}),

					displayfield({
						cls: "right label-left border-bottom",
						name: "tasksTodo",
						label: t("Todo")
					}),


					displayfield({
						cls: "right label-left",
						name: "tasksTotal",
						label: t("Total")
					}),
				),




				fieldset({legend: "Invoiced", cls: "vbox gap"},

					comp({tagName:"canvas" , id: "project-finance-billable-chart"}),

					displayfield({
						cls: "right",
						name: "quoteAmount",
						label: t("Quote"),
						renderer:(v) => {
							return Format.money(v);
						}
					}),

					displayfield({
						cls: "right border-bottom",
						name: "billableFee",
						label: t("Hours"),
						renderer:(v) => {
							return Format.money(v);
						}
					}),

					displayfield({
						cls: "right",
						name: "billableTotal",
						label: t("Total"),
						renderer:(v) => {
							return Format.money(v);
						}
					}),

					displayfield({
						cls: "right border-bottom",
						name: "invoicedAmount",
						label: t("Invoiced"),
						renderer:(v) => {
							return Format.money(-v);
						}
					}),

					displayfield({
						cls: "right",
						name: "outstanding",
						label: t("Outstanding"),
						renderer:(v) => {
							return Format.money(v);
						}
					}),

					comp({flex: 1}), // fill space till bottom in hbox layout

					btn({
						text: t("Create invoice"),
						cls: "outlined",
						handler: async () => {

							const fv = this.form.value, invoice = await jmapds("FinanceDocument").single(this.documentId!.toString());
							const win = new InvoiceWindow(invoice, fv);
							win.show();
						}
					})



				),

				fieldset({legend: "Profit", cls: "vbox gap"},

					comp({tagName:"canvas" , id: "project-finance-profit-chart"}),

					displayfield({
						cls: "right",
						name: "quoteAmount",
						label: t("Quote amount"),
						renderer:(v) => {
							return Format.money(v);
						}
					}),

					displayfield({
						cls: "right",
						name: "quoteCosts",
						label: t("Quote costs"),
						renderer:(v) => {
							return Format.money(-v);
						}
					}),

					displayfield({
						cls: "right",
						name: "billableFee",
						label: t("Billable fee"),
						renderer:(v) => {
							return Format.money(v);
						}
					}),

					displayfield({
						cls: "right border-bottom",
						name: "feeCosts",
						label: t("Fee cost"),
						renderer:(v) => {
							return Format.money(-v);
						}
					}),

					displayfield({
						cls: "right",
						name: "profit",
						label: t("Profit"),
						renderer:(v) => {
							return Format.money(v);
						}
					}),

				),

				// displayfield({
				// 	name: "fee",
				// 	label: t("Fee"),
				// 	renderer:(v) => {
				// 		return Format.money(v);
				// 	}
				// }),
				//
				// displayfield({
				// 	name: "quotedFee",
				// 	label: t("Quoted fee"),
				// 	renderer:(v) => {
				// 		return Format.money(v);
				// 	}
				// })
				)
			)
		)
	}

	private drawHoursChart(newValue:any) {
		const data = {
			labels: [
				'Actual',
				'Billable'
			],
			centerLabel: newValue.quotedMinutes ? Math.ceil((newValue.bookedMinutes / newValue.quotedMinutes) * 100) + "%" : "-",
			datasets: [{
				data: newValue.quotedMinutes ? [newValue.bookedMinutes / 60, Math.max(0, newValue.quotedMinutes - newValue.bookedMinutes) / 60] : [newValue.bookedMinutes / 60],
				// backgroundColor: [
				// 	'rgb(255, 99, 132)',
				// 	'rgb(54, 162, 235)',
				// 	'rgb(255, 205, 86)'
				// ],
				hoverOffset: 4
			}]
		};

		new Chart("project-finance-hour-chart", {
			type: 'doughnut',
			data: data,
			options: {
				responsive: true,
				plugins: {
					legend: {
						display: false
					},
					//doughnutCenterLabel: {}
				}
			}
		});
	}

	private drawTaskChart(newValue:any) {

		const data = {
			labels: [
				'Completed',
				'Todo'
			],
			centerLabel: newValue.tasksTotal ? Math.ceil((newValue.tasksCompleted / newValue.tasksTotal) * 100) + "%" : "-",
			datasets: [{
				data: [newValue.tasksCompleted, newValue.tasksTodo],
				// backgroundColor: [
				// 	'rgb(255, 99, 132)',
				// 	'rgb(54, 162, 235)',
				// 	'rgb(255, 205, 86)'
				// ],
				hoverOffset: 4
			}]
		};

		new Chart("project-finance-task-chart", {
			type: 'doughnut',
			data: data,
			options: {
				responsive: true,
				plugins: {
					legend: {
						display: false
					},
					//doughnutCenterLabel: {}
				}
			}
		});
	}


	private drawBillableChart(newValue:any) {
		const data = {
			labels: [
				'Invoiced of quote',
				'Billed fee',
				'Invoicable for quote',
				'Billable fee',
			],
			centerLabel: Math.ceil((100 - (newValue.outstanding / (newValue.quoteAmount + newValue.billableFee)) * 100)) + "%",
			datasets: [{
				data: [
					newValue.invoicedAmount,
					newValue.billedFee,
					newValue.quoteAmount - newValue.invoicedAmount,
					newValue.billableFee - newValue.billedFee,

				],
				backgroundColor: [
					'rgb(54, 162, 235)',
					'rgb(54, 162, 235)',
					'rgb(255, 99, 132)',
					'rgb(255, 99, 132)',

				],
				hoverOffset: 4
			}]
		};

		new Chart("project-finance-billable-chart", {
			type: 'doughnut',
			data: data,
			options: {
				responsive: true,
				plugins: {
					legend: {
						display: false
					},
					//doughnutCenterLabel: {}
				}
			}
		});
	}

	private drawProfitChart(newValue:any) {

		const margin = Math.ceil((1 - (newValue.quoteCosts + newValue.feeCosts) / (newValue.quoteAmount + newValue.billableFee)) * 100);

		const data = {
			labels: [
				'Profit',
				'Billable fee',
				'Quote costs',
				'Fee costs',
			],
			centerLabel: margin + "%",
			datasets: [{
				data: [
					newValue.quoteAmount - newValue.quoteCosts - newValue.feeCosts,
					newValue.billableFee,
					newValue.quoteCosts,
					newValue.feeCosts
				],
				backgroundColor: [
					'rgb(54, 162, 235)',
					'rgb(54, 162, 235)',
					'rgb(255, 99, 132)',
					'rgb(255, 99, 132)',

				],
				hoverOffset: 4
			}]
		};

		new Chart("project-finance-profit-chart", {
			type: 'doughnut',
			data: data,
			options: {
				responsive: true,
				plugins: {
					legend: {
						display: false
					},
					//doughnutCenterLabel: {}
				}
			}
		});
	}


	public async loadProject(projectId:number, documentId:number) {
		this.projectId = projectId;
		this.documentId = documentId;
		this.form.value = await client.jmap("FinanceDocument/getProjectData", {projectId: projectId, documentId: documentId});;
	}
}