import { BaseEntity, Button, CardContainer, Component, DefaultEntity, EntityID, Listener, ObservableListenerOpts, Toolbar, Window, WindowEventMap } from "@intermesh/goui";
import { SharePanel } from "../permissions";
import { CreateLinkButton } from "./CreateLinkButton";
/**
 * @inheritDoc
 */
export interface FormWindowEventMap<Type> extends WindowEventMap<Type> {
    /**
     * Fires when the window is shown and loaded with data. Also fires if the dialog is for creating new entities and
     * not loaded.
     *
     * @param window
     */
    ready: (window: Type, currentId: EntityID | undefined) => void;
    addlink: (window: Type, entityName: string, entityId: EntityID) => void;
}
export interface FormWindow<EntityType extends BaseEntity = DefaultEntity> {
    on<K extends keyof FormWindowEventMap<this>, L extends Listener>(eventName: K, listener: Partial<FormWindowEventMap<this>>[K], options?: ObservableListenerOpts): L;
    un<K extends keyof FormWindowEventMap<this>>(eventName: K, listener: Partial<FormWindowEventMap<this>>[K]): boolean;
    fire<K extends keyof FormWindowEventMap<this>>(eventName: K, ...args: Parameters<FormWindowEventMap<any>[K]>): boolean;
}
export declare abstract class FormWindow<EntityType extends BaseEntity = DefaultEntity> extends Window {
    entityName: string;
    readonly form: import("@intermesh/goui").DataSourceForm<EntityType>;
    protected readonly cards: CardContainer;
    protected sharePanel?: SharePanel;
    protected bbar: Toolbar;
    /**
     * The first tab
     *
     * @protected
     */
    protected readonly generalTab: Component;
    private cardMenu;
    protected readonly createLinkBtn: CreateLinkButton;
    /**
     * Enable this for linkable entities. It will show a create link button for new items
     */
    hasLinks: boolean;
    protected submitBtn: Button;
    /**
     * Constructor
     *
     * @param entityName Name of the entity for the datasource form
     * @protected
     */
    protected constructor(entityName: string);
    protected onBeforeClose(byUser: boolean): boolean | undefined;
    protected onShow(): void;
    /**
     * Add a share panel to set permissions
     *
     * @params options if not provided the default is:
     * [
     * 	{value: "", name: ""},
     * 	{value: 10, name: t("Read")},
     * 	{value: 20, name: t("Create")},
     * 	{value: 30, name: t("Write")},
     * 	{value: 40, name: t("Delete")},
     * 	{value: 50, name: t("Manage")}
     * ]
     */
    protected addSharePanel(levels?: {
        [key: string]: any;
    }[]): void;
    load(id: EntityID): Promise<this>;
    protected addCustomFields(): void;
    private renderCustomFields;
    /**
     * Adds a link between two entities on save.
     *
     * @param entityName - The name of the target entity.
     * @param  entityId - The ID of the target entity.
     */
    addLinkOnSave(entityName: string, entityId: string): void;
}
//# sourceMappingURL=FormWindow.d.ts.map