import { AutocompleteField } from "./AutocompleteField.js";
import { datasourcestore, } from "../../data/index.js";
import { column, table } from "../table/index.js";
import { createComponent } from "../Component.js";
import { Format } from "../../util/index.js";
import { t } from "../../Translate";
export const ComboBoxDefaultRenderer = (field, r) => r ? r[field.displayProperty] : t("Not found");
/**
 * Combo box
 *
 * An extension of the Autocomplete field that simplifies the creation of a simple combobox with a
 * valueProperty and displayProperty.
 */
export class ComboBox extends AutocompleteField {
    constructor(dataSource, displayProperty = "name", valueProperty = "id", renderer = ComboBoxDefaultRenderer, storeConfig = {
        queryParams: {
            limit: 50
        }
    }) {
        storeConfig.dataSource = dataSource;
        const dropDownTable = table({
            headers: false,
            fitParent: true,
            store: datasourcestore(storeConfig),
            columns: [
                column({
                    id: displayProperty,
                    resizable: true,
                    width: 312,
                    sortable: true,
                    renderer: (_columnValue, record) => {
                        return Format.escapeHTML(renderer(this, record));
                    }
                })
            ]
        });
        super(dropDownTable);
        this.dataSource = dataSource;
        this.displayProperty = displayProperty;
        this.valueProperty = valueProperty;
        this.renderer = renderer;
        /**
         * When autocompleting from the datasource this filter name will be used.
         */
        this.filterName = "text";
        // load next data on scroll
        this.picker.list.store.addScrollLoader(this.menu.el);
        this.picker.list.store.queryParams.limit = 50;
        this.on("autocomplete", async (_field, input) => {
            this.list.store.queryParams.position = 0;
            if (!this.list.store.queryParams.filter) {
                this.list.store.queryParams.filter = {};
            }
            const filterName = (this.filterName ?? this.displayProperty);
            this.list.store.setFilter(filterName, { [filterName]: input });
            this.list.store.setFilter("combo", this.filter);
            await this.list.store.load();
        });
    }
    pickerRecordToValue(_field, record) {
        return record[this.valueProperty];
    }
    async valueToTextField(_field, value) {
        if (value === "") {
            return "";
        }
        const record = await this.dataSource.single(value);
        return this.renderer(_field, record);
    }
}
/**
 * Shorthand function to create {@see ComboBox}
 *
 * @link https://goui.io/#form/Select
 * @param config
 */
export const combobox = (config) => createComponent(new ComboBox(config.dataSource, config.displayProperty ?? "name", config.valueProperty ?? "id", config.renderer ?? ComboBoxDefaultRenderer, config.storeConfig ?? {
    queryParams: {
        limit: 50
    }
}), config);
//# sourceMappingURL=ComboBox.js.map